from http import HTTPStatus
from io import BytesIO
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import Client
from ...types import UNSET, File, Response, Unset


def _get_kwargs(
    project_name: str,
    *,
    client: Client,
    experiments: Union[Unset, None, str] = UNSET,
    favorite: Union[Unset, None, bool] = False,
) -> Dict[str, Any]:
    url = "{}/projects/{projectName}/_export_models".format(client.base_url, projectName=project_name)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    params["experiments"] = experiments

    params["favorite"] = favorite

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, client: Client, response: httpx.Response) -> Optional[File]:
    if response.status_code == HTTPStatus.OK:
        response_200 = File(payload=BytesIO(response.content))

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(f"Unexpected status code: {response.status_code}")
    else:
        return None


def _build_response(*, client: Client, response: httpx.Response) -> Response[File]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    project_name: str,
    *,
    client: Client,
    experiments: Union[Unset, None, str] = UNSET,
    favorite: Union[Unset, None, bool] = False,
) -> Response[File]:
    """export models of the project

    Args:
        project_name (str):
        experiments (Union[Unset, None, str]):
        favorite (Union[Unset, None, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[File]
    """

    kwargs = _get_kwargs(
        project_name=project_name,
        client=client,
        experiments=experiments,
        favorite=favorite,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    project_name: str,
    *,
    client: Client,
    experiments: Union[Unset, None, str] = UNSET,
    favorite: Union[Unset, None, bool] = False,
) -> Optional[File]:
    """export models of the project

    Args:
        project_name (str):
        experiments (Union[Unset, None, str]):
        favorite (Union[Unset, None, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[File]
    """

    return sync_detailed(
        project_name=project_name,
        client=client,
        experiments=experiments,
        favorite=favorite,
    ).parsed


async def asyncio_detailed(
    project_name: str,
    *,
    client: Client,
    experiments: Union[Unset, None, str] = UNSET,
    favorite: Union[Unset, None, bool] = False,
) -> Response[File]:
    """export models of the project

    Args:
        project_name (str):
        experiments (Union[Unset, None, str]):
        favorite (Union[Unset, None, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[File]
    """

    kwargs = _get_kwargs(
        project_name=project_name,
        client=client,
        experiments=experiments,
        favorite=favorite,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    project_name: str,
    *,
    client: Client,
    experiments: Union[Unset, None, str] = UNSET,
    favorite: Union[Unset, None, bool] = False,
) -> Optional[File]:
    """export models of the project

    Args:
        project_name (str):
        experiments (Union[Unset, None, str]):
        favorite (Union[Unset, None, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[File]
    """

    return (
        await asyncio_detailed(
            project_name=project_name,
            client=client,
            experiments=experiments,
            favorite=favorite,
        )
    ).parsed



from typing import TYPE_CHECKING, Any, Dict, Type, TypeVar, Union

import attr

from ..models.imported_doc_category_creation_mode import ImportedDocCategoryCreationMode
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.imported_doc_category_properties import ImportedDocCategoryProperties


T = TypeVar("T", bound="ImportedDocCategory")


@attr.s(auto_attribs=True)
class ImportedDocCategory:
    """
    Attributes:
        creation_mode (Union[Unset, ImportedDocCategoryCreationMode]): Creation mode
        label (Union[Unset, str]): Human-friendly label
        label_id (Union[Unset, str]): External label identifier
        label_name (Union[Unset, str]): Label name
        properties (Union[Unset, ImportedDocCategoryProperties]): Additional properties
        score (Union[Unset, float]): Score of the category
        status (Union[Unset, str]): Status
    """

    creation_mode: Union[Unset, ImportedDocCategoryCreationMode] = UNSET
    label: Union[Unset, str] = UNSET
    label_id: Union[Unset, str] = UNSET
    label_name: Union[Unset, str] = UNSET
    properties: Union[Unset, "ImportedDocCategoryProperties"] = UNSET
    score: Union[Unset, float] = UNSET
    status: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        creation_mode: Union[Unset, str] = UNSET
        if not isinstance(self.creation_mode, Unset):
            creation_mode = self.creation_mode.value

        label = self.label
        label_id = self.label_id
        label_name = self.label_name
        properties: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.properties, Unset):
            properties = self.properties.to_dict()

        score = self.score
        status = self.status

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if creation_mode is not UNSET:
            field_dict["creationMode"] = creation_mode
        if label is not UNSET:
            field_dict["label"] = label
        if label_id is not UNSET:
            field_dict["labelId"] = label_id
        if label_name is not UNSET:
            field_dict["labelName"] = label_name
        if properties is not UNSET:
            field_dict["properties"] = properties
        if score is not UNSET:
            field_dict["score"] = score
        if status is not UNSET:
            field_dict["status"] = status

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.imported_doc_category_properties import ImportedDocCategoryProperties

        d = src_dict.copy()
        _creation_mode = d.pop("creationMode", UNSET)
        creation_mode: Union[Unset, ImportedDocCategoryCreationMode]
        if isinstance(_creation_mode, Unset):
            creation_mode = UNSET
        else:
            creation_mode = ImportedDocCategoryCreationMode(_creation_mode)

        label = d.pop("label", UNSET)

        label_id = d.pop("labelId", UNSET)

        label_name = d.pop("labelName", UNSET)

        _properties = d.pop("properties", UNSET)
        properties: Union[Unset, ImportedDocCategoryProperties]
        if isinstance(_properties, Unset):
            properties = UNSET
        else:
            properties = ImportedDocCategoryProperties.from_dict(_properties)

        score = d.pop("score", UNSET)

        status = d.pop("status", UNSET)

        imported_doc_category = cls(
            creation_mode=creation_mode,
            label=label,
            label_id=label_id,
            label_name=label_name,
            properties=properties,
            score=score,
            status=status,
        )

        return imported_doc_category



from typing import Any, Dict, List, Type, TypeVar, cast

import attr

T = TypeVar("T", bound="ExternalDatabases")


@attr.s(auto_attribs=True)
class ExternalDatabases:
    """
    Attributes:
        databases (List[str]):
    """

    databases: List[str]

    def to_dict(self) -> Dict[str, Any]:
        databases = self.databases

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "databases": databases,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        databases = cast(List[str], d.pop("databases"))

        external_databases = cls(
            databases=databases,
        )

        return external_databases
